#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <math.h>


#ifndef _MAX_PATH
#define _MAX_PATH 260
#endif

#define GUI 1

#define USE_XML 1


#if not GUI
int nop() {return 0;};
#define cvShowImage(x,y) cvSaveImage(x ".png",y)
#define cvWaitKey(x) nop()
#define cvNamedWindow(x,y) nop()
#endif

static int nscale=4;

static int m_flag = 0;
static IplImage* m_normal_img = 0;
static IplImage* m_scaled_img = 0;
static int m_road_num = 1;
static int m_draw_mode = 0;
static CvPoint m_prev_pt;



void create_normal_mouse_callback(int event, int x, int y, int flags )
{
    if( event == CV_EVENT_LBUTTONDOWN )
    {
        m_flag = 1;   
        m_prev_pt = cvPoint(x,y);

	printf("<point x=\"%d\" y=\"%d\" op=\"eq\" color=\"#%02X%.02X%.02X\"/>\n",x,y, 
          (unsigned char)m_normal_img->imageData[y*m_normal_img->widthStep+3*x+2],
          (unsigned char)m_normal_img->imageData[y*m_normal_img->widthStep+3*x+1],
          (unsigned char)m_normal_img->imageData[y*m_normal_img->widthStep+3*x+0]);
    }
    else if( event == CV_EVENT_LBUTTONUP )
    {  
        m_flag = 0;
        if( m_draw_mode == 1 )
        {
        //    cvLine( m_bg_img, m_prev_pt, cvPoint(x,y), cvScalar(m_road_num), m_brush_radius, -1 );
        }
    }    
    else if( event == CV_EVENT_MOUSEMOVE )
    {  
    }
    else if( event == CV_EVENT_RBUTTONDOWN )
    { 
        m_road_num++;
    }
}

void create_scaled_mouse_callback(int event, int x, int y, int flags )
{
    x/=nscale;
    y/=nscale;
    create_normal_mouse_callback(event, x,y,flags);
}

//road map mannual marking
int  create_road_map( char* image_file_name )
{
    IplImage*  rm = 0;

    char       fn[_MAX_PATH];
    long       h_file;
    IplImage*  img = 0;
    IplImage*  scale_img = 0;
    int        r = 0, k = 0;
    
    //printf("Loading %s\n",image_file_name);
    img = cvLoadImage( image_file_name, 1 );
    if(img==NULL) 
    {
      fprintf(stderr,"Could not load image: %s!\n",image_file_name);
      exit(0);
    }
    //printf("img: %p\n",img);
   
    cvNamedWindow("normal", 1);    
    cvNamedWindow("scaled", 1);    

    //printf("Showing ...\n");
    cvShowImage("normal", img);

    //printf("Scaling ...\n");
    CvSize mysize=cvSize(img->width*nscale,img->height*nscale);

    scale_img=cvCreateImage( mysize,IPL_DEPTH_8U, 3 );
    cvResize(img,scale_img,CV_INTER_NN);
    //printf("Showing ...\n");
    cvShowImage("scaled", scale_img);

    cvSetMouseCallback( "normal", create_normal_mouse_callback ); 
    cvSetMouseCallback( "scaled", create_scaled_mouse_callback ); 

    for(;;)
    {
        m_normal_img = img;
        m_scaled_img = scale_img;

        while(1)
        {
            for(;;)
            {
                k = cvWaitKey(0);
                if( k == 27 ) exit(0);
                else if( k == 's' )
		{
//                    cvSaveImage( road_map_file_name, rm );
		}
                else if( k == 'c' )
                    m_draw_mode = 0;
                else if( k == 'l' )
                    m_draw_mode = 1;
                else
                    break;
            }

        }        
    }
    cvReleaseImage( &img );
    cvReleaseImage( &scale_img );

    return 1;
}

// usage: picking.exe <args> 
int main( int argc, char **argv )
{
    int mode = 0;
    if( argc < 1 )
    {
        printf( "Usage: %s\n  <filename.png>\n", argv[0] );
        return 0;
    }

    char* image_file_name = 0;
    create_road_map( argv[1] );

    return 1;
}

